@@ -7,18 +7,26 @@ open Graph
 (* command line *)
 let n_ = ref 30
 let prob_ = ref 0.5
+let seed_ = ref None
 
 let arg_spec = 
   ["-v", Arg.Int (fun i -> n_ := i), 
    " <int>  number of vertices";
    "-prob", Arg.Float (fun f -> prob_ := f), 
-   " <float>  probability to discrad an edge"
+   " <float>  probability to discrad an edge";
+   "-seed", Arg.Int (fun n -> seed_ := Some n),
+   " <int>  random seed"
   ]
 let () = Arg.parse arg_spec (fun _ -> ()) "usage: color <options>"
 
 let n = !n_
 let prob = !prob_
 
+let seed = match !seed_ with
+  | None -> Random.self_init (); Random.int (1 lsl 29)
+  | Some s -> s
+let () = Format.printf "seed = %d@." seed; Random.init seed
+
 (* undirected graphs with integer coordinates and integer labels on edges *)
 
 module IntInt = struct 
@@ -35,7 +43,6 @@ module G = Imperative.Graph.AbstractLabeled(IntInt)(Int)
 open G
 
 (* a random graph with n vertices *)
-let () = Random.self_init ()
 module R = Rand.Planar.I(G)
 let g0 = R.graph ~xrange:(20,780) ~yrange:(20,580) ~prob n
 
@@ -104,6 +111,14 @@ exception NoColor
 
 (* Algo I. Brute force. *)
 
+module C = Coloring.Mark(G)
+
+let coloring_a k =
+  Mark.clear g0;
+  C.coloring g0 4;
+  iter_vertex (fun v -> color_vertex v cols.(Mark.get v)) g0
+
+(***
 let coloring_a k =
   Mark.clear g0;
   (* first step: we eliminate vertices with less than 4 successors *)
@@ -135,7 +150,7 @@ let coloring_a k =
   if !nb_to_color > 0 then begin
     let rec iterate iter =
       let v = Bfs.get iter in
-      if Mark.get v = k+1 then
+      if Mark.get v > 0 then
 	iterate (Bfs.step iter)
       else begin
 	for i = 1 to k do
@@ -146,7 +161,7 @@ let coloring_a k =
 	raise NoColor
       end
     in
-    try iterate (Bfs.start g0); assert false with Exit -> ()
+    try iterate (Bfs.start g0) with Exit -> ()
   end;
   (* third step: we color the eliminated vertices, in reverse order *)
   Stack.iter
@@ -160,6 +175,7 @@ let coloring_a k =
     stack;
   (* finally we display the coloring *)  
   iter_vertex (fun v -> color_vertex v cols.(Mark.get v)) g0
+***)
 
 (* Algo II.
 
@@ -255,7 +271,7 @@ let coloring_b () =
 	raise NoColor
       end
     in
-    try iterate (Bfs.start g0); assert false with Exit -> ()
+    try iterate (Bfs.start g0) with Exit -> ()
   end;
   (* third step: we color the eliminated vertices, in reverse order *)
   Stack.iter
@@ -276,13 +292,26 @@ let coloring_b () =
        color_vertex v cols.(c)) 
     g0
 
+open Unix
+  
+let utime f x =                                                   
+  let u = (times()).tms_utime in                                  
+  let y = f x in
+  let ut = (times()).tms_utime -. u in
+  (y,ut)
+
+let print_utime f x = 
+  let (y,ut) = utime f x in
+  Format.printf "user time: %2.2f@." ut;
+  y
+
 let () =
   draw_graph ();
   (* test_bfs (); *)
   (* test_dfs (); *)
-  coloring_a 4;
-  ignore (Graphics.wait_next_event [ Key_pressed ]);
-  draw_graph ();
-  coloring_b ();
+  print_utime coloring_a 4;
+  (*ignore (Graphics.wait_next_event [ Key_pressed ]);*)
+  (*draw_graph ();*)
+  print_utime coloring_b ();
   ignore (Graphics.wait_next_event [ Key_pressed ]);
   close_graph ()
