@@ -15,11 +15,31 @@
  * (enclosed in the file LGPL).
  *)
 
-(* $Id: imperative.ml,v 1.21 2004/11/02 15:17:28 conchon Exp $ *)
+(* $Id: imperative.ml,v 1.25 2006/02/03 09:27:29 filliatr Exp $ *)
 
 open Sig
 open Per_imp
 
+module type S = sig
+
+  (** Imperative Unlabeled Graphs *)
+  module Concrete (V: COMPARABLE) : 
+    Sig.I with type V.t = V.t and type V.label = V.t and type E.t = V.t * V.t
+
+  (** Abstract Imperative Unlabeled Graphs *)
+  module Abstract(V: sig type t end) : Sig.IA with type V.label = V.t
+
+  (** Imperative Labeled Graphs *)
+  module ConcreteLabeled (V: COMPARABLE)(E: ORDERED_TYPE_DFT) :
+    Sig.I with type V.t = V.t and type V.label = V.t 
+	    and type E.t = V.t * E.t * V.t and type E.label = E.t
+
+  (** Abstract Imperative Labeled Graphs *)
+  module AbstractLabeled (V: sig type t end)(E: ORDERED_TYPE_DFT) :
+    Sig.IA with type V.label = V.t and type E.label = E.t
+
+end
+
 module I = Make(Make_Hashtbl)
 
 type 'a abstract_vertex = { tag : int; label : 'a; mutable mark : int }
@@ -90,6 +110,34 @@ module Digraph = struct
 
   end
 
+  module ConcreteBidirectional (V: COMPARABLE) = struct
+
+    include I.Digraph.ConcreteBidirectional(V)
+
+    type it = t
+    type iv = V.t
+    type ie = E.t
+
+    let add_vertex g v = if not (HM.mem v g) then unsafe_add_vertex g v
+
+    let add_edge g v1 v2 = 
+      add_vertex g v1;
+      add_vertex g v2;
+      unsafe_add_edge g v1 v2
+
+    let add_edge_e g (v1, v2) = add_edge g v1 v2
+
+    let remove_vertex g v =
+      if HM.mem v g then begin
+	iter_pred_e ( fun e -> remove_edge_e g e ) g v ;
+	iter_succ_e ( fun e -> remove_edge_e g e ) g v ;
+	HM.remove v g
+      end
+
+    let copy = HM.copy
+
+  end
+
   module ConcreteLabeled(V: COMPARABLE)(E: ORDERED_TYPE_DFT) = struct
 
     let default = E.default
@@ -236,7 +284,7 @@ module Graph = struct
 
     module G = Digraph.Concrete(V) 
 
-    include Graph.Concrete(G)
+    include Graph(G)
 
     (* Export some definitions of [G] *)
 
@@ -267,7 +315,7 @@ module Graph = struct
 
     module G = Digraph.ConcreteLabeled(V)(E)
 
-    include Graph.Concrete(G)
+    include Graph(G)
 
     (* Export some definitions of [G] *)
 
@@ -299,12 +347,9 @@ module Graph = struct
 
   module Abstract(V: sig type t end) = struct
 
-    module G = struct
-      include Digraph.Abstract(V)
-      let tag v = v.tag
-    end
+    module G = Digraph.Abstract(V)
 
-    include Graph.Abstract(G)
+    include Graph(G)
 
     (* Export some definitions of [G] *)
 
@@ -334,12 +379,9 @@ module Graph = struct
 
   module AbstractLabeled (V: sig type t end)(E: ORDERED_TYPE_DFT) = struct
 
-    module G = struct
-      include Digraph.AbstractLabeled(V)(E)
-      let tag v = v.tag
-    end
+    module G = Digraph.AbstractLabeled(V)(E)
 
-    include Graph.Abstract(G)
+    include Graph(G)
 
     (* Export some definitions of [G] *)
 
@@ -423,6 +465,7 @@ module Matrix = struct
     let mem_vertex g v = 0 <= v && v < nb_vertex g
     let mem_edge g i j = Bitv.get g.(i) j
     let mem_edge_e g (i,j) = Bitv.get g.(i) j
+    let find_edge g i j = if mem_edge g i j then i, j else raise Not_found
 			       
     (* constructors *)
     let add_edge g i j = Bitv.set g.(i) j true
@@ -553,7 +596,7 @@ module Matrix = struct
 
     module G = Digraph 
 
-    include Per_imp.Graph.Concrete(G)		 
+    include Per_imp.Graph(G)		 
     (* Export some definitions of [G] *)
 
     let create = G.create
