@@ -15,7 +15,7 @@
  * (enclosed in the file LGPL).
  *)
 
-(* $Id: per_imp.ml,v 1.28 2004/11/15 16:42:07 signoles Exp $ *)
+(* $Id: per_imp.ml,v 1.32 2006/02/03 09:27:29 filliatr Exp $ *)
 
 (* Common implementation to persistent and imperative graphs. *)
 
@@ -205,6 +205,8 @@ module Unlabeled(V: COMPARABLE)(HM: HM with type key = V.t) = struct
 
   let mem_edge_e g (v1, v2) = mem_edge g v1 v2
 
+  let find_edge g v1 v2 = if mem_edge g v1 v2 then v1, v2 else raise Not_found
+
   let unsafe_remove_edge g v1 v2 = HM.add v1 (S.remove v2 (HM.find v1 g)) g
   let unsafe_remove_edge_e g (v1, v2) = unsafe_remove_edge g v1 v2
 
@@ -241,12 +243,133 @@ module Unlabeled(V: COMPARABLE)(HM: HM with type key = V.t) = struct
 
 end
 
+(* Support for explicitly maintaining edge set of
+   predecessors.  Crucial for algorithms that do a lot of backwards
+   traversal. *)
+
+module BidirectionalMinimal(S:Set.S)(HM:HM with type key = S.elt) = 
+struct
+
+  type vertex = HM.key
+
+  let is_directed = true
+  let empty = HM.empty
+  let create = HM.create
+  let is_empty = HM.is_empty
+
+  let nb_vertex g = HM.fold (fun _ _ -> succ) g 0
+  let nb_edges g = HM.fold (fun _ (_,s) n -> n + S.cardinal s) g 0
+  let out_degree g v = 
+    S.cardinal (snd (try HM.find v g with Not_found -> invalid_arg "out_degree"))
+
+  let mem_vertex g v = HM.mem v g
+
+  let unsafe_add_vertex g v = HM.add v (S.empty,S.empty) g
+  let unsafe_add_edge g v1 v2 = 
+    let (in_set,out_set) = HM.find v1 g
+    in 
+      ignore ( HM.add v1 (in_set,S.add v2 out_set) g ) ;
+      let (in_set,out_set) = HM.find v2 g
+      in
+	HM.add v2 (S.add v1 in_set,out_set) g
+
+  let iter_vertex f = HM.iter (fun v _ -> f v)
+  let fold_vertex f = HM.fold (fun v _ -> f v)
+end
+
+module BidirectionalUnlabeled(V:COMPARABLE)(HM:HM with type key = V.t) =
+struct
+
+  module S = Set.Make(V)
+
+  (* Edge definition *)
+
+  module E = struct
+    type vertex = V.t
+    include OTProduct(V)(V)
+    let src = fst
+    let dst = snd
+    type label = unit
+    let label _ = ()
+    let create v1 () v2 = v1, v2
+  end
+
+  type edge = E.t
+
+  let mem_edge g v1 v2 =
+    try S.mem v2 (snd (HM.find v1 g))
+    with Not_found -> false
+
+  let mem_edge_e g (v1,v2) = mem_edge g v1 v2
+
+  let find_edge g v1 v2 = if mem_edge g v1 v2 then v1, v2 else raise Not_found
+
+  let unsafe_remove_edge g v1 v2 =
+    let (in_set,out_set) = HM.find v1 g in 
+    ignore ( HM.add v1 (in_set,( S.remove v2 out_set )) g ) ;
+    let (in_set,out_set) = HM.find v2 g in
+    HM.add v2 (S.remove v1 in_set,out_set) g
+
+  let unsafe_remove_edge_e g (v1,v2) = unsafe_remove_edge g v1 v2
+
+  let remove_edge g v1 v2 = 
+    if not (HM.mem v2 g) then invalid_arg "remove_edge";
+    unsafe_remove_edge g v1 v2
+    (* HM.add v1 (S.remove v2 (HM.find_and_raise v1 g "remove_edge")) g *)
+
+  let remove_edge_e g (v1, v2) = remove_edge g v1 v2
+
+  let iter_succ f g v = S.iter f (snd (HM.find_and_raise v g "iter_succ"))
+  let fold_succ f g v = S.fold f (snd (HM.find_and_raise v g "fold_succ"))
+
+  let iter_succ_e f g v = iter_succ (fun v2 -> f (v, v2)) g v
+  let fold_succ_e f g v = fold_succ (fun v2 -> f (v, v2)) g v
+
+  let succ g v = S.elements (snd (HM.find_and_raise v g "succ"))
+  let succ_e g v = fold_succ_e (fun e l -> e :: l) g v []
+ 
+  let map_vertex f = 
+    HM.map (fun v (s1,s2) -> 
+	      f v, 
+	      ( S.fold (fun v s -> S.add (f v) s) s1 S.empty,
+		S.fold (fun v s -> S.add (f v) s) s2 S.empty ) 
+	   )
+
+  module I = struct
+    (* we keep sets for both incoming and outgoing edges *)
+    type t = (S.t * S.t) HM.t  
+    module PV = V
+    module PE = E
+    let iter_edges f = HM.iter (fun v (_,outset) -> S.iter (f v) outset )
+    let fold_edges f = HM.fold (fun v (_,outset) -> S.fold (f v) outset )
+    let iter_edges_e f = iter_edges (fun v1 v2 -> f (v1, v2))
+    let fold_edges_e f = fold_edges (fun v1 v2 a -> f (v1, v2) a)
+  end
+  include I
+
+  let iter_pred f g v = S.iter f (fst (HM.find_and_raise v g "iter_pred"))
+  let fold_pred f g v = S.fold f (fst (HM.find_and_raise v g "fold_pred"))
+
+  let pred g v = S.elements (fst (HM.find_and_raise v g "pred"))
+
+  let in_degree g v = 
+    S.cardinal 
+      (fst (try HM.find v g with Not_found -> invalid_arg "in_degree"))
+
+  let iter_pred_e f g v = iter_pred (fun v2 -> f (v2,v)) g v
+  let fold_pred_e f g v = fold_pred (fun v2 -> f (v2,v)) g v
+
+  let pred_e g v = fold_pred_e (fun e l -> e :: l) g v []   
+end
+
 (* Common implementation to all the labeled (directed) graphs. *)
 module Labeled(V: COMPARABLE)(E: ORDERED_TYPE)(HM: HM with type key = V.t) = 
 struct
-  
-  module S = Set.Make(OTProduct(V)(E))
-    
+
+  module VE = OTProduct(V)(E)
+
+  module S = Set.Make(VE)
+
   module E = struct
     type vertex = V.t
     type label = E.t
@@ -255,9 +378,9 @@ struct
     let dst (_, _, v) = v
     let label (_, l, _) = l
     let create v1 l v2 = v1, l, v2
-    module C = OTProduct(V)(OTProduct(E)(V))
+    module C = OTProduct(V)(VE)
     let compare (x1, x2, x3) (y1, y2, y3) = 
-      C.compare (x1, (x2, x3)) (y1, (y2, y3))
+      C.compare (x1, (x3, x2)) (y1, (y3, y2))
   end
 
   type edge = E.t
@@ -268,7 +391,22 @@ struct
     with Not_found ->
       false
 
-  let mem_edge_e g (v1, _, v2) = mem_edge g v1 v2
+  let mem_edge_e g (v1, l, v2) =
+    try
+      let ve = v2, l in
+      S.exists (fun ve' -> VE.compare ve ve' == 0) (HM.find v1 g)
+    with Not_found ->
+      false
+
+  exception Found of edge
+  let find_edge g v1 v2 =
+    try
+      S.iter 
+	(fun (v2', l) -> if V.equal v2 v2' then raise (Found (v1, l, v2')))
+	(HM.find v1 g);
+      raise Not_found
+    with Found e ->
+      e
 
   let unsafe_remove_edge g v1 v2 = 
     HM.add v1 (S.filter 
@@ -303,7 +441,7 @@ struct
   let map_vertex f = 
     HM.map (fun v s -> 
 	      f v, S.fold (fun (v, l) s -> S.add (f v, l) s) s S.empty)
-    
+
   module I = struct
     type t = S.t HM.t
     module PV = V
@@ -392,6 +530,7 @@ module Make_Abstract(G: Sig.G) = struct
   let mem_vertex g = G.mem_vertex g.edges
   let mem_edge g = G.mem_edge g.edges
   let mem_edge_e g = G.mem_edge_e g.edges
+  let find_edge g = G.find_edge g.edges
 
 (* JS: factorisation de remove_edge impossible due  un bug caml;
    laisser le code en commentaire en attendant un fix *)
@@ -424,6 +563,12 @@ module Make(F : functor(X: COMPARABLE) -> HM with type key = X.t) = struct
 
     end
 
+    module ConcreteBidirectional(V: COMPARABLE) = struct
+      include ConcreteVertex(F)(V)
+      include BidirectionalUnlabeled(V)(HM)
+      include BidirectionalMinimal(S)(HM)
+    end
+
     module ConcreteLabeled(V: COMPARABLE)(E: ORDERED_TYPE_DFT) = struct
 
       let default = E.default
@@ -511,128 +656,45 @@ module Make(F : functor(X: COMPARABLE) -> HM with type key = X.t) = struct
 
 end
 
-module Graph = struct
+(* Implementation of undirected graphs from implementation of directed graphs.
+ *)
+module Graph(G: Sig.G) = struct
 
-  (* Predecessors are the successors in an undirected graph. *)
-  module Pred(S: sig
-		module PV : VERTEX
-		module PE : EDGE
-		type t
-		val succ : t -> PV.t -> PV.t list
-		val out_degree : t -> PV.t -> int
-		val iter_succ : (PV.t -> unit) -> t -> PV.t -> unit
-		val fold_succ : (PV.t -> 'a -> 'a) -> t -> PV.t -> 'a -> 'a
-		val succ_e : t -> PV.t -> PE.t list
-		val iter_succ_e : (PE.t -> unit) -> t -> PV.t -> unit
-		val fold_succ_e : (PE.t -> 'a -> 'a) -> t -> PV.t -> 'a -> 'a
-	      end) = 
-  struct
-    open S
-    let pred = succ
-    let in_degree = out_degree
-    let iter_pred = iter_succ
-    let fold_pred = fold_succ
-    let pred_e = succ_e
-    let iter_pred_e = iter_succ_e
-    let fold_pred_e = fold_succ_e
-  end
+  include G
 
-  (* Same functor for unlabeled and labeled concrete graphs *)
-  module Concrete(G: Sig.G) = struct
+  let is_directed = false
 
-    module M = struct
+  (* Optimise the edges iterators. *)
 
-      include G
-      module PV = V
-      module PE = E
-		    
-      let is_directed = false
-			  
-      (* Optimise the edges iterators. *)
-			  
-      module H = Hashtbl.Make(HTProduct(G.V)(G.V))
+  let iter_edges f =
+    iter_edges (fun v1 v2 -> if V.compare v1 v2 >= 0 then f v1 v2)
 
-      let iter_edges f =
-	let h = H.create 97 in
-	G.iter_edges 
-	  (fun v1 v2 -> 
-	     if not (H.mem h (v1, v2)) then begin 
-	       H.add h (v2, v1) ();
-	       f v1 v2
-	     end) 
-	    
-      let fold_edges f =
-	let h = H.create 97 in
-	G.fold_edges 
-	  (fun v1 v2 acc -> 
-	     if not (H.mem h (v1, v2)) then begin 
-	       H.add h (v2, v1) ();
-	       f v1 v2 acc
-	     end else
-	       acc) 
+  let fold_edges f =
+    fold_edges 
+      (fun v1 v2 acc -> if V.compare v1 v2 >= 0 then f v1 v2 acc else acc)
 
-      let iter_edges_e f =
-	let h = H.create 97 in
-	G.iter_edges_e
-	  (fun e ->
-	     let v1, v2 as vv = G.E.src e, G.E.dst e in
-	     if not (H.mem h vv) then begin 
-	       H.add h (v2, v1) ();
-	       f e
-	     end) 
+  let iter_edges_e f =
+    iter_edges_e
+      (fun e -> if V.compare (E.src e) (E.dst e) >= 0 then f e)
 
-      let fold_edges_e f =
-	let h = H.create 97 in
-	G.fold_edges_e
-	  (fun e acc -> 
-	     let v1, v2 as vv = G.E.src e, G.E.dst e in
-	     if not (H.mem h vv) then begin 
-	       H.add h (v2, v1) ();
-	       f e acc
-	     end else
-	       acc)
-	  
-    end
+  let fold_edges_e f =
+    fold_edges_e
+      (fun e acc -> 
+	 if V.compare (E.src e) (E.dst e) >= 0 then f e acc else acc)
 
-    include M
-    include Pred(M)
+  (* Redefine [nb_edges] *)
 
-  end
+  let nb_edges g = fold_edges_e (fun _ -> (+) 1) g 0
 
-  (* Same functor for unlabeled and labeled abstract graphs *)
-  module Abstract(G: sig include Sig.G val tag : V.t -> int end) = struct
+  (* Redefine operations on predecessors:
+     predecessors are successors in an undirected graph. *)
 
-    module M = struct
-      include G
-
-      module PV = V
-      module PE = E
-
-      let is_directed = false
-
-      (* Optimise the edges iterators. 
-	 Better optimisations than the concrete ones. *)
-
-      let iter_edges f =
-	iter_edges (fun v1 v2 -> if tag v1 <= tag v2 then f v1 v2)
-	  
-      let fold_edges f =
-	fold_edges 
-	  (fun v1 v2 acc -> if tag v1 <= tag v2 then f v1 v2 acc else acc)
-
-      let iter_edges_e f =
-	iter_edges_e (fun e -> if tag (E.src e) <= tag (E.dst e) then f e)
-
-      let fold_edges_e f =
-	fold_edges_e 
-	  (fun e acc -> 
-	     if tag (E.src e) <= tag (E.dst e) then f e acc else acc)
-	  
-    end
-
-    include M
-    include Pred(M)
-
-  end
+  let pred = succ
+  let in_degree = out_degree
+  let iter_pred = iter_succ
+  let fold_pred = fold_succ
+  let pred_e = succ_e
+  let iter_pred_e = iter_succ_e
+  let fold_pred_e = fold_succ_e
 
 end
