@@ -15,12 +15,32 @@
  * (enclosed in the file LGPL).
  *)
 
-(* $Id: persistent.ml,v 1.16 2004/11/25 15:27:05 conchon Exp $ *)
+(* $Id: persistent.ml,v 1.18 2005/01/17 15:22:03 signoles Exp $ *)
 
 open Sig
 open Util
 open Per_imp
 
+module type S = sig
+
+  (** Persistent Unlabeled Graphs *)
+  module Concrete (V: COMPARABLE) : 
+    Sig.P with type V.t = V.t and type V.label = V.t and type E.t = V.t * V.t
+
+  (** Abstract Persistent Unlabeled Graphs *)
+  module Abstract(V: sig type t end) : Sig.P with type V.label = V.t
+
+  (** Persistent Labeled Graphs *)
+  module ConcreteLabeled (V: COMPARABLE)(E: ORDERED_TYPE_DFT) :
+    Sig.P with type V.t = V.t and type V.label = V.t
+	    and type E.t = V.t * E.t * V.t and type E.label = E.t
+
+  (** Abstract Persistent Labeled Graphs *)
+  module AbstractLabeled (V: sig type t end)(E: ORDERED_TYPE_DFT) :
+    Sig.P with type V.label = V.t and type E.label = E.t
+
+end
+
 module P = Make(Make_Map)
 
 type 'a abstract_vertex = { tag : int; label : 'a }
@@ -176,7 +196,7 @@ module Graph = struct
 
     module G = Digraph.Concrete(V) 
 
-    include Graph.Concrete(G)
+    include Graph(G)
 
     (* Export some definitions of [G] *)
 
@@ -206,7 +226,7 @@ module Graph = struct
 
     module G = Digraph.ConcreteLabeled(V)(E)
 
-    include Graph.Concrete(G)
+    include Graph(G)
 
     (* Export some definitions of [G] *)
 
@@ -237,12 +257,9 @@ module Graph = struct
 
   module Abstract(V: sig type t end) = struct
 
-    module G = struct
-      include Digraph.Abstract(V)
-      let tag v = v.tag
-    end
+    module G = Digraph.Abstract(V)
 
-    include Graph.Abstract(G)
+    include Graph(G)
 
     (* Export some definitions of [G] *)
 
@@ -270,12 +287,9 @@ module Graph = struct
 
   module AbstractLabeled (V: sig type t end)(E: ORDERED_TYPE_DFT) = struct
 
-    module G = struct
-      include Digraph.AbstractLabeled(V)(E)
-      let tag v = v.tag
-    end
+    module G = Digraph.AbstractLabeled(V)(E)
 
-    include Graph.Abstract(G)
+    include Graph(G)
 
     (* Export some definitions of [G] *)
 
