@@ -15,7 +15,7 @@
  * (enclosed in the file LGPL).
  *)
 
-(* $Id: traverse.mli,v 1.13 2004/11/04 13:10:28 filliatr Exp $ *)
+(* $Id: traverse.mli,v 1.14 2005/04/01 07:13:24 filliatr Exp $ *)
 
 (** Graph traversal *)
 
@@ -39,16 +39,18 @@ module Dfs(G : G) : sig
              ?post:(G.V.t -> unit) -> G.t -> unit
       (** [iter pre post g] visits all nodes of [g] in depth-first search, 
 	 applying [pre] to each visited node before its successors,
-	 and [post] after them. Each node is visited exactly once. *)
+	 and [post] after them. Each node is visited exactly once. 
+         Not tail-recursive. *)
 
   val prefix : (G.V.t -> unit) -> G.t -> unit
     (** applies only a prefix function; note that this function is more
-      efficient than [iter] *)
+      efficient than [iter] and is tail-recursive. *)
 
   val postfix : (G.V.t -> unit) -> G.t -> unit
-    (** applies only a postfix function *)
+    (** applies only a postfix function. Not tail-recursive. *)
 
-  (** Same thing, but for a single connected component *)
+  (** Same thing, but for a single connected component
+      (only [prefix_component] is tail-recursive) *)
 
   val iter_component : ?pre:(G.V.t -> unit) -> 
              ?post:(G.V.t -> unit) -> G.t -> G.V.t -> unit
